import os.path
import time
from xml.etree import ElementTree as ET
import unicodedata

LEVEL_LENGTH = 3


def chunks(l, n):
    """Split list l to chunks of size n."""
    return [l[i:i+n] for i in range(0, len(l), n)]


def normalize(text):
    """Normalize strings for comparing answers."""
    return remove_accents(text.strip().lower())


def remove_accents(input_str):
    if not isinstance(input_str, unicode):
        return input_str
    nkfd_form = unicodedata.normalize('NFKD', input_str)
    return u"".join([c for c in nkfd_form if not unicodedata.combining(c)])


class Quiz(object):
    def __init__(self, title, levels):
        self.title = title
        self.levels = levels


class Level(object):
    def __init__(self, questions):
        self.questions = questions

    def __iter__(self):
        return iter(self.questions)

    def __getitem__(self, item):
        return self.questions[item]

    def check_answers(self, answers):
        """
        Check answers for all questions in the level.
        Return a number of wrong answers.
        """
        errors = len(self.questions) - len(answers)
        for question, answer in zip(self.questions, answers):
            if not question.check_answer(answer):
                errors += 1
        return errors


class Question(object):
    def __init__(self, text, answer):
        self.text = text
        self.answer = normalize(answer)

    def check_answer(self, answer):
        return normalize(answer) == self.answer


class QuizzesStore(object):
    def __init__(self, directory):
        self.base_dir = directory

    def _read_quiz_element(self, quiz_id):
        try:
            quizzes_file = os.path.join(self.base_dir, quiz_id + '.xml')
            tree = ET.parse(quizzes_file)
            return tree.getroot()
        except IOError:
            raise QuizNotFound()


    def _read_questions(self, level_elem):
        questions = []
        for question_elem in level_elem.iter('question'):
            answer_elem = question_elem.find('answer')
            answer = answer_elem.text
            question_elem.remove(answer_elem)
            q_text = ET.tostring(question_elem, 'utf-8', 'html')
            q_text = q_text.replace('<question>', '').replace('</question>', '').strip()
            questions.append(Question(q_text, answer))
        
        return questions

    def load(self, quiz_id):
        quiz_elem = self._read_quiz_element(quiz_id)
        levels = []
        level_elements = quiz_elem.findall('level')
        if len(level_elements) > 0:  # Levels present
            for level_elem in level_elements:
                questions = self._read_questions(level_elem)
                levels.append(Level(questions))
        else:  # Implicit levels
            questions = self._read_questions(quiz_elem)
            for chunk in chunks(questions, LEVEL_LENGTH):
                levels.append(Level(chunk))
            
        return Quiz(quiz_elem.findtext('title', ''), levels)


class QuizNotFound(Exception):
    pass


# Hall of Fame contains information about users who solved quizzes.
#
# Hall of fame is stored in SQLite3 database and all functions for working with
# it require opened database connection as a first parameter.


def read_hof(db, quiz_id):
    """
    Read hall-of-fame for a specified quiz.
    Returns a list of dictionaries (name, time).
    """
    cur = db.execute("select name, time from hof where quiz=?", (quiz_id,))
    hof = [{'name': r[0], 'time': r[1]} for r in cur]
    cur.close()
    return hof


def append_to_hof(db, quiz_id, name):
    """Append an hall-of-fame record with current time"""
    db.execute("insert into hof(quiz,name,time) values (?,?,?)",
               (quiz_id, name, time.time()))
    db.commit()
