import sqlite3

from flask import Flask, abort, g, jsonify, render_template, request, session, url_for

from model import QuizzesStore, QuizNotFound, read_hof, append_to_hof
import os
from utils import jsonp


SOLVED = 'solved'

# default settings
DEBUG = True
SECRET_KEY = 'development key'
QUIZZES_DIR = "quizzes"
DATABASE = 'hof.db'
QUIZZES_PRODUCERS = ['127.0.0.1']

# application initialization
app = Flask(__name__)
app.config.from_object(__name__)
app.config.from_envvar('QUIZZES_SETTINGS', silent=True)

# Quizzes store
quizzes = QuizzesStore(app.config['QUIZZES_DIR'])


# Database setup #############################################################

def init_db():
    with app.app_context():
        db = get_db()
        with app.open_resource('schema.sql', mode='r') as f:
            db.cursor().executescript(f.read())
        db.commit()


def get_db():
    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = sqlite3.connect(app.config['DATABASE'])
    return db


@app.teardown_appcontext
def close_connection(exception):
    db = getattr(g, '_database', None)
    if db is not None:
        db.close()


# Views ######################################################################

@app.route('/<quiz_id>', methods=['GET'])
@jsonp
def get_questions(quiz_id):
    try:
        quiz = quizzes.load(quiz_id)
        level = current_level(quiz_id)
        questions = [question.text for question in quiz.levels[level]]
        return jsonify(title=quiz.title, level=level, levels=len(quiz.levels),
                       questions=questions)
    except QuizNotFound:  # Quiz not found
        abort(404)
    except IndexError:  # Invalid level number in session
        session[quiz_id] = 0
        abort(500)


@app.route('/<quiz_id>/check', methods=['GET'])
@jsonp
def check_answers(quiz_id):
    try:
        quiz = quizzes.load(quiz_id)
        level = current_level(quiz_id)
        form = request.args if request.method == "GET" else request.form
        errors = quiz.levels[level].check_answers(form.getlist('quiz'))
        finish = False
        if errors == 0:
            session[quiz_id] += 1
            if level + 1 == len(quiz.levels):  # Finished!
                finish = True
                session[quiz_id] = SOLVED
        return jsonify(errors=errors, finish=finish)
    except QuizNotFound:  # Quiz not found
        abort(404)
    except IndexError:  # Invalid level number in session
        session[quiz_id] = 0
        abort(500)


@app.route('/<quiz_id>', methods=['PUT'])
def upload_quiz(quiz_id):
    if request.remote_addr in app.config['QUIZZES_PRODUCERS']:
        f = request.files['quiz']
        if f is not None:
            f.save(os.path.join(app.config['QUIZZES_DIR'], quiz_id + '.xml'))
            return "OK"
        else:
            abort(400)
    else:
        abort(403)


@app.route('/<quiz_id>/hof', methods=['GET'])
@jsonp
def hall_of_fame(quiz_id):
    hof = read_hof(get_db(), quiz_id)
    return jsonify(hof=hof)


@app.route('/<quiz_id>/hof/add', methods=['GET'])
@jsonp
def add_to_hall_of_fame(quiz_id):
    if quiz_id not in session and session[quiz_id] != SOLVED:
        return jsonify(status="ERROR")  # Quiz was not solved yet
    form = request.args if request.method == "GET" else request.form
    if 'name' in form:
        name = form['name'].strip()
        if name != "":
            append_to_hof(get_db(), quiz_id, name)
        return jsonify(status="OK")
    else:
        return jsonify(status="ERROR")


@app.route('/<quiz_id>/embed')
def embed(quiz_id):
    return render_template('embed.html', quiz_id=quiz_id)


@app.route('/embed.js')
def embed_js():
    quiz_id = request.args.get('id', '')
    return render_template('embed.js', embed_url=url_for('embed', quiz_id=quiz_id, _external=True))


def current_level(quiz_id):
    """Get current level number from the session."""
    if quiz_id not in session:
        session[quiz_id] = 0
    if session[quiz_id] == SOLVED:
        session[quiz_id] = 0
    level = session[quiz_id]
    return level


if __name__ == "__main__":
    app.run()
