"use strict";

/**
 * Setup quizzes in the current page.
 *
 * There must by a <div> element in the document with attribute data-quiz-id set
 * to quiz identifier. This element would be used to place quiz form.
 *
 * @param {jQuery} elements   jQuery collection of quiz elements.
 * @param {String} serverUrl  URL of the quiz server.
 */
function setupQuizzes(elements, serverUrl) {
    "use strict";
    elements.each(function () {
        setupQuiz($(this), serverUrl);
    });
}

/**
 * Setup quiz in the current page.
 *
 * There must by a <div> element in the document with attribute data-quiz-id set
 * to quiz identifier. This element would be used to place quiz form.
 *
 * @param {jQuery} rootElement  Quiz element selected with jQuery.
 * @param {String} serverUrl    URL of the quiz server.
 */
function setupQuiz(rootElement, serverUrl) {
    "use strict";
    var id = rootElement.data('quizId'), // Quiz ID
    // Quiz UI components
        title, switcherLabel,
        questionsPanel = new QuestionsPanel(),
        hofPanel = new HallOfFamePanel(),
        panel = 'hof', // Currently displayed panel
    // Quiz state
        level = 0, // Current level in the quiz
        finish = false; // Was quiz finished?

    // Setup quiz
    prepareForm(rootElement);
    switchPanel(); // Switch to quiz panel at start

    /**
     * Prepare quiz UI with two panels: "question form" and "hall of fame".
     * @param {jQuery} root  Root element of the quiz
     */
    function prepareForm(root) {
        root.addClass('quiz-frame');

        var panelSwitcher = $('<a href="#" class="hof pseudo"/>');
        panelSwitcher.on('click', switchPanel);
        switcherLabel = $('<small>').appendTo(panelSwitcher);

        title = $('<strong class="title"/>');

        var panelsArea = $('<div/>');

        root.append(panelSwitcher, title, panelsArea);
        panelsArea.append(questionsPanel.panel, hofPanel.panel)
    }

    /* Switch between two panels of the quiz form */
    function switchPanel() {
        if (panel == 'quiz') {
            panel = 'hof';
            questionsPanel.panel.hide();
            hofPanel.panel.show();
            switcherLabel.text('« späť na otázky');
            readHof();
        } else {
            panel = 'quiz';
            hofPanel.panel.hide();
            questionsPanel.panel.show();
            switcherLabel.text('kto už vyriešil');
            readQuestions();
        }
        return false;
    }

    /** Read quiz questions from the server. */
    function readQuestions() {
        $.getJSON(serverUrl + id + '?callback=?', function (data) {
            level = data.level;
            title.text(data.title);
            questionsPanel.displayQuestions(data.questions);
        });
    }

    /**
     * Submit form to the quiz server.
     * Depending on the value of finish, it submits answers to quiz questions
     * or a name of the user to the Hall of Fame.
     */
    function submit() {
        if (finish) {
            $.getJSON(serverUrl + id + '/hof/add?callback=?',
                questionsPanel.serializeForm(),
                function () {
                    finish = false;
                    switchPanel();
                });
        } else {
            $.getJSON(serverUrl + id + '/check?callback=?',
                questionsPanel.serializeForm(),
                function (data) {
                    if (data.errors > 0) {
                        questionsPanel.showMessage(
                                'Nesprávne. Počet chýb: ' + data.errors);
                    } else if (data.errors === 0) {
                        finish = data.finish;
                        if (finish) {
                            questionsPanel.askName();
                        } else {
                            readQuestions();
                        }
                    }
                });
        }
        return false;
    }

    /** Get Hall of Fame data and display them. */
    function readHof() {
        $.getJSON(serverUrl + id + '/hof?callback=?', function (data) {
            hofPanel.displayTable(data.hof);
        });
        return false;
    }


    /**
     * UI component for displaying quiz questions.
     * @constructor
     */
    function QuestionsPanel() {
        var form = $('<form/>');
        var questionsArea = $('<div class="questions"/>');
        var submitArea = $('<div/>');
        form.append(questionsArea, submitArea);
        var messageArea = $('<p class="quiz-message"/>');
        var submitButton = $('<input type="submit">').click(submit);
        submitArea.append(messageArea, submitButton);

        /** Main UI panel */
        this.panel = form;

        /**
         * Display quiz questions to the user.
         * @param {Array} quiz  Quiz questions
         */
        this.displayQuestions = function (quiz) {
            questionsArea.empty();
            for (var i = 0; i < quiz.length; i++) {
                var input = $('<input>', {'name': 'quiz',
                    'autocomplete': 'off'});
                $('<div/>')
                    .append($('<p/>', {'html': quiz[i]}))
                    .append(input)
                    .appendTo(questionsArea);
                if (level > 0 && i === 0) input.focus();
            }
            $('<input type="hidden">').attr({'name': 'level', 'value': level})
                .appendTo(questionsArea);
            messageArea.html('');
            if (typeof MathJax !== undefined) {
                MathJax.Hub.Queue(["Typeset", MathJax.Hub, questionsArea.get(0)]);
            }
        };

        /**
         * Show message in the message area.
         * @param {String} text  Message text.
         */
        this.showMessage = function (text) {
            messageArea.html(text);
        };

        /**
         * Get serialized version of quiz answers ready for sending to the server.
         * @returns {String}
         */
        this.serializeForm = function () {
            return form.serialize();
        };

        /**
         * Display message about the solved quiz and ask user their name,
         * so they can be added to the Hall of Fame.
         */
        this.askName = function () {
            messageArea.html($('<a/>', {'href': '#', 'class': 'pseudo',
                'text': 'Zostať v anonymite', 'click': function () {
                    finish = false;
                    switchPanel();
                    return false;
                }}));
            questionsArea.empty();
            var input = $('<input/>', {'id': 'quiz-user-name', 'name': 'name'});
            $('<p/>', {'text': 'Blahoželám! Úspešne ste vyriešili kvíz.'})
                .appendTo(questionsArea);
            $('<p/>')
                .append($('<label/>', {'text': 'Vaše meno: ',
                    'for': 'quiz-user-name'}))
                .append(input)
                .appendTo(questionsArea);
            input.focus();
        };
    }


    /**
     * UI component for displaying Hall of Fame listing.
     * @constructor
     */
    function HallOfFamePanel() {
        /** Main UI panel */
        this.panel = $('<div/>').hide();
        var table = $('<table/>').appendTo(this.panel);

        /**
         * Display Hall of Fame table
         * @param {Array} hof
         */
        this.displayTable = function (hof) {
            table.empty();
            for (var i = 0; i < hof.length; i++) {
                var date = new Date(hof[i].time * 1000);
                table.append($('<tr/>')
                    .append($('<td/>', {text: hof[i].name}))
                    .append($('<td/>', {text: date.toLocaleDateString() + " " +
                        date.toLocaleTimeString()})));
            }
        }
    }
}
